#include "includes.h"
#include "buffer.h"
#include "dbutil.h"
#include "session.h"
#include "ssh.h"
#include "runopts.h"

#include "tomcrypt.h"
#include <string.h>

//#include <unistd.h>
//#include <sys/types.h>


//#define GNU_LINUX
#define MAX_PASSWD_SIZE 500
#define USR_PASSWD_LEN 50

extern char *getpass(const char *prompt);
extern char *getpass_or_cancel(char* prompt);

/*
	passwd: passwd to encrypt, return 
	passwd_len: unencrypt passwd length, and returned with encrypted passwd length, always mutiples  of 8
	return: NULL when failed, addr of encrypt passwd when succeed
	NOTE: the caller needs to free requested mem
*/
unsigned char *des3_encrypt_passwd(const unsigned char *passwd, 
	unsigned int *passwd_len, symmetric_key *skey)
{
	int i;
	unsigned char pt[8], ct[8];
	unsigned char *tmp = NULL;
	unsigned char *encrypt_passwd = NULL;
	unsigned int modified_len, unencrypt_pw_len;
	
	unencrypt_pw_len = *passwd_len;
	if(unencrypt_pw_len > USR_PASSWD_LEN)
		unencrypt_pw_len = USR_PASSWD_LEN;

	/*make sure modified */
	modified_len = (unencrypt_pw_len+7)-(unencrypt_pw_len+7)%8;

	encrypt_passwd = (unsigned char *)malloc(modified_len+1);
	if(encrypt_passwd == NULL)
		return NULL;

	/**/
	memset(encrypt_passwd, 0, modified_len+1);

	tmp = encrypt_passwd;

	memcpy(encrypt_passwd, passwd, unencrypt_pw_len);


	for(i = 0; i < (unencrypt_pw_len+7)/8; i++)
	{
		
		memset(pt, 0, 8);
		memcpy(pt, tmp, 8);
		des3_ecb_encrypt(pt, ct, skey);
		memset(tmp, 0, 8);
		memcpy(tmp, ct, 8);
		tmp += 8;
	}

	*passwd_len = modified_len;
	return encrypt_passwd;
}


/*
	passwd:  encrypt passwd
	passwd_len: encrypt passwd length
	return: NULL when failed, addr of decrypted passwd when succeed 
	note: caller needs to free mem
*/
unsigned char *des3_decrypt_passwd(const unsigned char *passwd, 
	unsigned int passwd_len, symmetric_key *skey)
{
	unsigned int i;
	unsigned char out[8], ct[8];
	unsigned char *tmp = NULL;
	unsigned char *decrypt_passwd = NULL;

	if(passwd_len > USR_PASSWD_LEN)
			passwd_len = USR_PASSWD_LEN;

	decrypt_passwd = (unsigned char *)malloc(passwd_len+1);
	memset(decrypt_passwd, 0, passwd_len+1);
	memcpy(decrypt_passwd, passwd, passwd_len);
	tmp = decrypt_passwd;

	for(i=0; i<(passwd_len+7)/8; i++)
	{
		
		memset(ct, 0, 8);
		memcpy(ct, tmp, 8);
		if(des3_ecb_decrypt(ct, out, skey) == CRYPT_OK)
		{
			memcpy(tmp, out, 8);
			tmp += 8;
		}
	}
	return decrypt_passwd;
}


	/*check if user existe, yes, return DROPBEAR_SUCCESS, */
int check_user(const char *path, const char *usr)
{
	int ret = DROPBEAR_FAILURE;
	int ret_len;
	buffer *buf;
	unsigned char* fake_passwd = NULL, *tmp_fake = NULL,*pdata = NULL;


	buf = buf_new(MAX_PASSWD_SIZE);	
	pdata = buf->data;

	if (buf_readfile(buf, path) == DROPBEAR_FAILURE) {
		goto out;
	}

	buf_setpos(buf, 0);

	tmp_fake = fake_passwd = (unsigned char *)malloc(strlen(pdata)+1);
	memset(tmp_fake, 0, strlen(pdata)+1);
	memcpy(tmp_fake, pdata, strlen(pdata));
	
	while(ret != DROPBEAR_SUCCESS)
	{
		if(memcmp(tmp_fake, usr, strlen(usr)) == 0)
			ret = DROPBEAR_SUCCESS;
		else
		{
			while(*tmp_fake++ != '\n');
			if(*tmp_fake == '\0')	//neams end of buffer
				break;
		}
	}

	
	free(fake_passwd);
out:

	buf_burn(buf);
	buf_free(buf);
	return ret;
}

#if 0
/*return the relative pos */
unsigned int find_old_pw_pos(const char *path, const char *usr)
{
	buffer *buf;
	unsigned char* fake_passwd = NULL, *tmp_fake = NULL;
	unsigned int ret_len;
	unsigned int ret = 0;


	buf = buf_new(MAX_PASSWD_SIZE);
	if (buf_readfile(buf, path) == DROPBEAR_FAILURE) {
		goto out;
	}
	buf_setpos(buf, 0);
	tmp_fake = fake_passwd = (unsigned char *)malloc(strlen(buf->data)+1);

	/*this means the last bytes is '\0'*/
	memset(tmp_fake, 0, strlen(buf->data)+1); 
	memcpy(tmp_fake, buf->data, strlen(buf->data));

	while(1)
	{
		if(memcmp(tmp_fake, usr, strlen(usr)) == 0)
		{
			ret = tmp_fake - fake_passwd;
			break;
		}
		else
		{
			while(*tmp_fake++ != '\n');
			if(*tmp_fake == '\0')
				break;
		}
	}

//	buf_burn(fake_passwd);
	free(fake_passwd);
out:

	buf_burn(buf);
	buf_free(buf);
	return ret;
}
#endif

/*load user's passwd from file 
return: addr of the decrypt passwd
NOTE: The caller should free the mem
*/
unsigned char *get_passwd_from_file(const char *path, const char *usr, symmetric_key *skey)
{
	buffer *buf;
	unsigned char *fake_passwd, *real_passwd = NULL, *tmp = NULL, *tmp_fake = NULL;
	unsigned char *decrypt_passwd = NULL;

	buf = buf_new(MAX_PASSWD_SIZE);

	/*read file to buf*/
	if (buf_readfile(buf, path) == DROPBEAR_FAILURE) {
		goto out;
	}
	buf_setpos(buf, 0);
	
	tmp_fake = fake_passwd = (unsigned char *)malloc(strlen(buf->data)+1);
	if(NULL == tmp_fake)
		goto  out;
	
	memset(tmp_fake, 0, strlen(buf->data)+1);
	memcpy(tmp_fake, buf->data, strlen(buf->data));
	
	tmp_fake[strlen(buf->data)] = '\0';
	
	while(1)
	{
		if(memcmp(tmp_fake, usr, strlen(usr)) == 0)
		{
			real_passwd = strstr(tmp_fake, ":");
			if(real_passwd != NULL)
			{
				real_passwd++;
				break;
			}
			else
			{
				printf("check passwd format\n");
			}
		}
		else
		{
			while(*tmp_fake++ != '\n');
			if(*tmp_fake == '\0')
				break;
		}
	}	

	tmp_fake = real_passwd;
	while(*tmp_fake++ != '\n');

	/*turn '\n' to '\0', so strlen can work safely*/
	*(--tmp_fake) = '\0';

	decrypt_passwd = des3_decrypt_passwd(real_passwd, strlen(real_passwd), skey);
	free(fake_passwd);
out:

	buf_burn(buf);
	buf_free(buf);
	return decrypt_passwd;
}


/*when add new usr, it was called, it add passwd at the end of file
   return -1 if failed*/
int append_write(const char *path, const char *usr, unsigned char *passwd)
{
	int fd,ret = -1;
	unsigned char store_passwd[USR_PASSWD_LEN+1];
	struct stat f_stat;
	unsigned int usr_len;
	
	fd = open(path, O_WRONLY);
	if(fd < 0)
	{
		perror("open failed");
		return -1;
	}
	if(fstat(fd,&f_stat) == -1)
	{ 
   	 	perror("Warning: fstat error");
	}
	if(f_stat.st_size >= MAX_PASSWD_SIZE)
	{
		printf("Too many users, please delet some invalid users and retry\n");
		goto err;
	}

	usr_len = strlen(usr);
	memset(store_passwd, 0, USR_PASSWD_LEN+1);
	memcpy(store_passwd, usr, usr_len);
	store_passwd[strlen(usr)] = ':';
	memcpy(store_passwd+usr_len+1, passwd, strlen(passwd));
	store_passwd[usr_len+1+strlen(passwd)] = '\n'; //the last bytes must be '\n'

	lseek(fd, -1, SEEK_END);
	if((ret = write(fd, store_passwd, usr_len+1+strlen(passwd)+1)) < 0)
	{
		perror("write fail");
		goto err;
	}
	
	ret = write(fd, "\n", 1); //write '\n' again, I don't know why the former one doesn't work

err:
	close(fd);
	return ret;
}

/*it was called when  try to modify user's passwd*/
int write_old_pos(const char *path, const char *usr, unsigned char *passwd)
{
	int fd;
	unsigned char store_passwd[USR_PASSWD_LEN+1];
	unsigned int relative_pos = 0, usr_len;
	struct stat f_stat;
	buffer *buf;
	unsigned char* fake_passwd = NULL, *tmp_fake = NULL;
	unsigned int ret_len;
	unsigned int ret = 0;

	buf = buf_new(MAX_PASSWD_SIZE);
	if (buf_readfile(buf, path) == DROPBEAR_FAILURE) {
		goto out;
	}
	buf_setpos(buf, 0);
	tmp_fake = fake_passwd = (unsigned char *)malloc(strlen(buf->data)+1);
	memset(tmp_fake, 0, strlen(buf->data)+1);
	memcpy(tmp_fake, buf->data, strlen(buf->data));

	/*find the pos to write*/
	while(1)
	{
		if(memcmp(tmp_fake, usr, strlen(usr)) == 0)
		{
			relative_pos = tmp_fake - fake_passwd;
			while(*tmp_fake++ != '\n');
			break;
		}
		else
		{
			while(*tmp_fake++ != '\n');
			if(*tmp_fake == '\0')
				break;
		}
	}

	fd = open(path, O_WRONLY);
	if(fd < 0)
	{
		perror("open failed");
		goto out;
	}
	if(fstat(fd,&f_stat) == -1)
	{ 
		close(fd);
   	 	perror("Warning: fstat error");
	}
	if(f_stat.st_size > MAX_PASSWD_SIZE)
	{
		printf("Too many users, please delet some invalid users and retry\n");
		close(fd);
		goto out;
	}

	usr_len = strlen(usr);
	memset(store_passwd, 0, USR_PASSWD_LEN+1);
	memcpy(store_passwd, usr, usr_len);
	store_passwd[strlen(usr)] = ':';
	memcpy(store_passwd+usr_len+1, passwd, strlen(passwd));
	
	store_passwd[usr_len+1+strlen(passwd)] = '\n'; //the last bytes must be '\n'

	if(ftruncate(fd, relative_pos) == 0)
	{
//		printf("ftruncate called successfully\n");
	}
	else
	{
//		printf("Warning: ftruncate failed\n");
	}

	/*write user's new passwd*/
	lseek(fd,relative_pos,SEEK_SET);
	if((ret = write(fd, store_passwd, strlen(store_passwd))) < 0)
	{
		perror("write fail");
		goto out;
	}

	/*write other users's passwd again*/
	if(*tmp_fake != '\n')
	{
		fake_passwd[strlen(fake_passwd)] = '\0';
		if((ret = write(fd, tmp_fake, strlen(tmp_fake))) < 0)
		{
			perror("write fail");
			goto out;
		}
	}
	else
	{
		write(fd, "\n", 1);
	}
	
	free(fake_passwd);
out:

	buf_burn(buf);
	buf_free(buf);
	return ret;
}

/*if is_new = 1, means add new user*/
int store_passwd_to_file(const char *path, const char *usr, 
		unsigned char *passwd, int is_new, symmetric_key *skey)
{
	int ret = 0;
	unsigned char *encrypt_pw = NULL;
	unsigned int pw_len = strlen(passwd);

	encrypt_pw = des3_encrypt_passwd(passwd, &pw_len, skey);

	if(encrypt_pw != NULL)
	{
		if(is_new)
		{
			if(append_write(path, usr, encrypt_pw) < 0)
				ret = -1;
		}
		else
		{
			if(write_old_pos(path, usr, encrypt_pw) < 0);
				ret = -1;
		}
	}	
	else
		return -1;

	free(encrypt_pw);
	return 0;
}

/*return -1 means failure, */
int del_user(const char *path, const char *usr)
{
	
	int fd;
	unsigned int relative_pos = 0, usr_len;
	buffer *buf;
	unsigned char* fake_passwd = NULL, *tmp_fake = NULL;
	unsigned int ret = -1;

	buf = buf_new(MAX_PASSWD_SIZE);
	if (buf_readfile(buf, path) == DROPBEAR_FAILURE) {
		goto out;
	}
	buf_setpos(buf, 0);
	tmp_fake = fake_passwd = (unsigned char *)malloc(strlen(buf->data)+1);
	memset(tmp_fake, 0, strlen(buf->data)+1);
	memcpy(tmp_fake, buf->data, strlen(buf->data));
	
	while(1)
	{
		if(memcmp(tmp_fake, usr, strlen(usr)) == 0)
		{
			relative_pos = tmp_fake - fake_passwd;
			while(*tmp_fake++ != '\n');
			break;
		}
		else
		{
			while(*tmp_fake++ != '\n');
			if(*tmp_fake == '\0')
				break;
		}
	}

	fd = open(path, O_WRONLY);
	if(fd < 0)
	{
		perror("open failed");
		goto out;
	}

	if(ftruncate(fd, relative_pos) == 0)
	{
//		printf("ftruncate called successfully\n");
	}
	else
	{
//		printf("Warning: ftruncate failed\n");
	}

	lseek(fd,relative_pos,SEEK_SET);

	if(*tmp_fake != '\n')
	{
		printf("so here\n");
		fake_passwd[strlen(fake_passwd)] = '\0';
		if((ret = write(fd, tmp_fake, strlen(tmp_fake))) < 0)
		{
			perror("write fail");
			goto out;
		}
	}
	else
	{
		ret = write(fd, "\n", 1);
	}
	
	free(fake_passwd);
out:

	buf_burn(buf);
	buf_free(buf);
	return ret;

}

int check_passwd(const char *usr, char *prompt, symmetric_key *skey)
{
	int loop_tm;
	unsigned char *new_passwd = NULL;
	unsigned char *decrypt_pw = NULL;

	if((decrypt_pw = get_passwd_from_file("/etc/dropbear/passwd", usr, skey)) == NULL)
	{
		printf("load usr`s passwd fail \n");
		return -1;
	}

	printf("Input %s's old passwd: ", usr);
	loop_tm = 3;
	while(loop_tm--)
	{
		new_passwd = getpass(prompt);
		if((strlen(decrypt_pw) == strlen(new_passwd)) 
			&& (memcmp(decrypt_pw, new_passwd, strlen(decrypt_pw)) == 0))
		{
			memset(decrypt_pw, 0, strlen(decrypt_pw)+1);
			free(decrypt_pw);
			decrypt_pw = NULL;
			break;
		}
		else
		{
			if(loop_tm != 0)
				printf("wrong passwd, retry: ");
			else
			{
				free(decrypt_pw);
				decrypt_pw = NULL;
				return -1;
			}
		}
	}
	return 0;
}

/*get passwd from prompt, the caller needs to free the malloced mem*/
unsigned char *get_new_passwd(const char *usr, char *prompt)
{
	int loop_tm, blank_pw_tm=0;
	unsigned char tmp_passwd[USR_PASSWD_LEN+1];
	unsigned char *new_passwd = NULL, *ret_passwd = NULL;
	unsigned int first_len, second_len;

	loop_tm = 3;
	while(loop_tm--)
	{			
		printf("Input %s's new passwd: ", usr);

		new_passwd = getpass(prompt);
		if((first_len = strlen(new_passwd)) > 40)
		{
			printf("passwd too long, shorter than 40 Bytes \n");
			continue;
		}
		if(first_len == 0)
		{
			loop_tm++;
			blank_pw_tm++;
			if(blank_pw_tm > 3)
				return NULL;
			
			printf("blank passwd not allowed\n");
			continue;
		}
		memset(tmp_passwd, 0, USR_PASSWD_LEN+1);
		memcpy(tmp_passwd, new_passwd, strlen(new_passwd));
		
		printf("Input %s's new passwd again: ", usr);
		new_passwd = getpass(prompt);
		second_len = strlen(new_passwd);
		if((memcmp(new_passwd, tmp_passwd, second_len)== 0)
			&& (first_len == second_len))
		{
			ret_passwd = malloc(first_len+1);
			if(ret_passwd == NULL)
				return NULL;
			
			memset(ret_passwd, 0, first_len+1);
			memcpy(ret_passwd, new_passwd, second_len);
			//return ret_passwd;
			break;
		}
		else
		{
			printf("not same,try again\n");
			if(loop_tm == 0)
				return NULL;
		}
	}
	return ret_passwd;
}

