/*
 * Dropbear - a SSH2 server
 * 
 * Copyright (c) 2002,2003 Matt Johnston
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE. */

/* Validates a user password */

#include "includes.h"
#include "session.h"
#include "buffer.h"
#include "dbutil.h"
#include "auth.h"
#include "dbpasswd.h"
//#include "nvram.h"

#ifdef ENABLE_SVR_PASSWORD_AUTH

#define PW_NVRAM_VALUE "2"

const char *encrypt_str = "ohyeahGrandSreamHZkehua6";
extern int des3_setup(const unsigned char *key, int keylen, int num_rounds, symmetric_key *skey);
//extern int des3_ecb_decrypt(const unsigned char *ct, unsigned char *pt, symmetric_key *skey);

/* Process a password auth request, sending success or failure messages as
 * appropriate */
void svr_auth_password() {
	
#ifdef HAVE_SHADOW_H
	struct spwd *spasswd = NULL;
#endif
	unsigned char * passwdcrypt = NULL; /* passwd from /etc/dropbear/passwd, decrypted*/
	char * testcrypt = NULL; /* crypt generated from the user's password sent */
	unsigned char *decrypt_pw = NULL;
	unsigned char * password;
	unsigned int passwordlen;
	unsigned int changepw;
	char *str = NULL;
	unsigned char key[24];
    symmetric_key skey;

// get password from nvram for admin and root both
#if defined GS_PRIVATE_DBCLIENT
	if (strncmp(ses.authstate.pw_name, "root", 4) == 0)
	{
		/*init 3des crypt*/
		memcpy(key, encrypt_str, 24);
		if (des3_setup(key, 24, 0, &skey) != CRYPT_OK) {
			send_msg_userauth_failure(0, 1);
			return ;
	    }
	
		/*get passwd from USER_PASSWD_PATH, instead of /etc/shadow, added by kehua*/
//		passwdcrypt = get_passwd_from_file(USER_PASSWD_PATH, ses.authstate.pw_name, &skey);
	}
	else
#endif
	{
		str = strdup(nvram_safe_get(PW_NVRAM_VALUE));
		passwdcrypt = str;
	}
#if 0
#ifdef HAVE_SHADOW_H
	/* get the shadow password if possible */
	spasswd = getspnam(ses.authstate.pw_name);
	if (spasswd != NULL && spasswd->sp_pwdp != NULL) {
		passwdcrypt = spasswd->sp_pwdp;
	}
#endif

#ifdef DEBUG_HACKCRYPT
	/* debugging crypt for non-root testing with shadows */
	passwdcrypt = DEBUG_HACKCRYPT;
#endif
#endif

	/* check if client wants to change password */
	changepw = buf_getbool(ses.payload);
	if (changepw) {
		/* not implemented by this server */
		send_msg_userauth_failure(0, 1);
		return;
	}

	password = buf_getstring(ses.payload, &passwordlen);

	/* the first bytes of passwdcrypt are the salt */
//	testcrypt = crypt((char*)password, passwdcrypt);

	/*decrypt passwd from client, addey by kehua*/
#if defined GS_PRIVATE_DBCLIENT
	if (strncmp(ses.authstate.pw_name, "root", 4) == 0)
		decrypt_pw = des3_decrypt_passwd(password, passwordlen, &skey);
	else
#endif
	{
		decrypt_pw = malloc(strlen(password) + 1);
		memset(decrypt_pw, 0, strlen(password) + 1);
		memcpy(decrypt_pw, password, strlen(password));
	}
//	printf("client passwd: %s\n", decrypt_pw);
//	printf("server passwd: %s\n", passwdcrypt);
	m_burn(password, passwordlen);
	m_free(password);

#if 0
	/* check for empty password */
	if (passwdcrypt[0] == '\0') {
#ifdef ALLOW_BLANK_PASSWORD
		if (passwordlen == 0) {
			success_blank = 1;
		}
#else
		dropbear_log(LOG_WARNING, "User '%s' has blank password, rejected",
				ses.authstate.pw_name);
		send_msg_userauth_failure(0, 1);
		return;
#endif
	}

#endif

/*compare passwds, added by kehua*/
	if(strlen(decrypt_pw) == strlen(passwdcrypt)
		&& (memcmp(decrypt_pw, passwdcrypt, strlen(passwdcrypt)) == 0))
	{
		dropbear_log(LOG_NOTICE, 
				"Password auth succeeded for '%s' from %s",
				ses.authstate.pw_name,
				svr_ses.addrstring);
		m_burn(decrypt_pw, strlen(decrypt_pw)+1);
		send_msg_userauth_success();
	} else {
		dropbear_log(LOG_WARNING,
				"Bad password attempt for '%s' from %s",
				ses.authstate.pw_name,
				svr_ses.addrstring);
		#ifdef GS_LOCK_SSH
		extern void gs_calc_error_times(void);
		gs_calc_error_times();
		#endif
		send_msg_userauth_failure(0, 1);
	}

#if 0
	if(strncmp(ses.authstate.pw_name, "root", 4) == 0)
	{
		free(passwdcrypt);
		passwdcrypt = NULL;
	}
#endif

//	printf("-----here 2-----\n");
	free(decrypt_pw);
	decrypt_pw = NULL;
        free(str);
        str = NULL;

}

#endif
