#!/system/xbin/sh

#############################################
#Policy -P App-Type:U:T:Vlan-ID:L2-Prio:DSCP
#############################################
#-------------------------------------------
#App-Type
#Valid application types:
# 1   Voice
# 2   Voice Signaling
# 3   Guest Voice
# 4   Guest Voice Signaling
# 5   Softphone Voice
# 6   Video Conferencing
# 7   Streaming Video
# 8   Video Signaling
#-------------------------------------------
#U   Unknown Policy Flag.
# 0   Network policy for the specified application type is defined
# 1   Network policy for the specified application type is required by the device but is currently unknown. This is used by Endpoint Devices, not by Network Connectivity Devices.
#-------------------------------------------
#T   Tagged Flag.
# 0   Untagged VLAN. In this case the VLAN ID and the Layer 2 Priority are ignored and only the DSCP value has relevance.
# 1   Tagged VLAN.
#-------------------------------------------
#VLAN-ID
#IEEE 802.1q VLAN ID (VID). A value of 1 through 4094 defines
#a VLAN ID. A value of 0 means that only the priority level is significant.
#-------------------------------------------
#L2-Prio
#IEEE 802.1d / IEEE 802.1p Layer 2 Priority, also known as Class of Service (CoS), 
#to be used for the specified application type.
# 1   Background
# 2   Spare
# 0   Best Effort (default)
# 3   Excellent Effort
# 4   Controlled Load
# 5   Video
# 6   Voice
# 7   Network Control
#-------------------------------------------
#DSCP
if [ -z "`nvram get 51`" -o -z "`nvram get 87`" ];then
	policy=1:0:0:0:0:0
else
	policy="1:0:1:`nvram get 51`:`nvram get 87`:0"
fi
################################################
#LLDP-MED POE-MDI -O type:source:priority:value
################################################
#-------------------------------------------
#type
#Valid types are:
#PSE   Power Sourcing Entity (power provider)
#PD    Power Device (power consumer)
#--------------------------------------------
#source
#Valid sources are:
# 0   Unknown
# 1   For PD, the power source is the PSE. For PSE, the power source is the primary power source.
# 2   For PD, the power source is a local source. For PSE, the power source is the backup power source 
#	  or a power conservation mode is asked (the PSE may be running on UPS for example).
# 3   For PD, the power source is both the PSE and a local source. For PSE, this value should not be used.
#-------------------------------------------
#priority
#Four priorities are available:
# 0   Unknown priority
# 1   Critical
# 2   High
# 3   Low
#-------------------------------------------
#value
#For PD, the power value is the total power in tenth of watts required by a PD device from the PSE device. 
#This value should range from 0 to 1023 tenth of watts.
#-------------------------------------------
if [ -z "`nvram get :med_poe_dtype`" -o -z "`nvram get :med_poe_source`" -o -z "`nvram get :med_poe_priority`" -o -z "`nvram get :med_poe_value`" ];then
	poe=PD:2:0:120
else
	poe="`nvram get :med_poe_dtype`:`nvram get :med_poe_source`:`nvram get :med_poe_priority`:`nvram get :med_poe_value`"
fi
##################################################
#Location -L
##################################################
#loc1=`nvram get :med_loc1`
#loc2=`nvram get :med_loc2`
#loc3=`nvram get :med_loc3`

#location=""
#for loc in $loc1 $loc2 $loc3
#do
#	if [ -n $loc ];then
#		location=${location}"-L $loc "
#	fi
#done

/system/bin/lldpctl -P $policy -O $poe &








