#!/system/xbin/sh
#@20111020
#@20111029 fix, apply different policy according to nvram.wan_ip_option
#@20111102 fix, pppoe_device=`wan_ifname`
#@20111108 fix, override dscp(nvram.38) with MED network policy's DSCP
#          lldpd disable old network policy first when LLDP packet changes.
#@20120202 fix, enalbe MED Network Policy TLVs use lldpctl 
#@20120607 remove (nvram.38) modification
#@20120926 fix static ip dns1 dns2 is not set bug,
#          GXP [Bug 12104] Phone loose VLAN tag randomly and loose the network connection.
# ANSI/TIA-1057- 2006
# ENABLE: 1 or 0
# DTYPE:
# 0-not defined
# 1-class I
# 2-class II
# 3-class III
# 4-Network Connectivity
# 5-255 Reserved
#
# ATYPE:
# 0-Reserved
# 1-Voice
# 2-Voice Signaling
# 3-Guest Voice
# 4-Guest Voice Signaling
# 5-Softphone Voice
# 6-Video Conferencing
# 7-Streaming Video
# 8-Video Signaling
# 9-255 Reserved
#
# UNKNOWN: 0 or 1
# TAGGED: 0 or 1
# VID: 0-Priority Tagged, 1-4094 VLAN ID, 4095 Reserved
# PRIORITY: 0-7
# DSCP: 0-63
#

ENABLE=$1
DTYPE=$2
ATYPE=$3
UNKNOWN=$4
TAGGED=$5
VID=$6
PRIORITY=$7
DSCP=$8

#echo "enable $ENABLE, dtype: $DTYPE, atype: $ATYPE, unknown: $UNKNOWN, tagged: $TAGGED, vid: $VID, priority: $PRIORITY, dscp: $DSCP" >> /tmp/lldp.log

if [ -z "$ENABLE" -o -z "$DTYPE" ];then
	exit 0
fi
if [ -z "$TAGGED" -o -z "$ATYPE" ];then
	exit 0
fi
if [ -z "$VID" -o -z "$PRIORITY" ];then
	exit 0
fi
if [ -z "$DSCP" -o -z "$UNKNOWN" ];then
	exit 0
fi

MED_NP="${ATYPE}:${UNKNOWN}:${TAGGED}:${VID}:${PRIORITY}:${DSCP}"
VLAN_PROP="net.vlan.tag"
LLDP_PROP="net.lldp.running"
VLAN_PRI_PROP="net.vlan.pri"
VLAN_OLD_PROP="net.vlan.old.tag"
VLAN_OLD_PRI_PROP="net.vlan.old.pri"
VLAN_OLD_AUDIO_DSCP="new.vlan.old.audio"
VLAN_OLD_SIP_DSCP="new.vlan.old.sip"
VLAN_OLD_VIDEO_DSCP="new.vlan.old.video"
NET_SAVE_PROP="net.qos.saved"
NET_STORE_PROP="net.qos.stored"

send_cfupdate()
{
    /system/xbin/dbus-send  --system --type=signal /com/grandstream/dbus/gui com.grandstream.dbus.signal.cfupdated
    echo "send cfupdated"  >> /tmp/lldp.log
}

med_need_join(){
	vlan_id=$1
	rval=0
    
	nv51=`nvram get 51`
	
	if [ ! -e /sys/class/net/eth0.${vlan_id} -a ! -e /sys/class/net/wlan0.${vlan_id} ]; then
		#VLAN ID is not equal to nvram.51 and VLAN interface does not exist
		rval=1
	fi

	echo "med vid ${vlan_id}, nvram 51=${nv51}, need join ${rval}" >> /tmp/lldp.log
	return $rval
}

med_need_leave(){
	vlan_id=$1
	rval=0

	if [ -e /sys/class/net/eth0.${vlan_id} -o -e /sys/class/net/wlan0.${vlan_id} ]; then
		#VLAN ID is not equal to nvram.51 and VLAN interface exist
		rval=1
	fi

	echo "med vid ${vlan_id}, need leave ${rval}" >> /tmp/lldp.log
	return $rval
}


store_cfg()
{
        #setprop ${NET_SAVE_PROP} "0"
	vlan_tag=`nvram get 51`
	qos_priority=`nvram get 87`
        sip_dscp=`nvram get 1558`
        audio_dscp=`nvram get 1559`
        video_dscp=`nvram get 1560`
        setprop $VLAN_OLD_PROP $vlan_tag
        setprop $VLAN_OLD_PRI_PROP $qos_priority
        setprop $VLAN_OLD_AUDIO_DSCP $audio_dscp
        setprop $VLAN_OLD_SIP_DSCP $sip_dscp
        setprop $VLAN_OLD_VIDEO_DSCP $video_dscp
        setprop $NET_STORE_PROP "1"
                
	echo "store vlan_tag=${vlan_tag}, qos_priority=${qos_priority},sip_dscp=${sip_dscp},audio_dscp=${audio_dscp},video_dscp=${video_dscp}"  >> /tmp/lldp.log
}

restore_cfg()
{
	vlan_tag=`getprop ${VLAN_OLD_PROP}`
	qos_priority=`getprop ${VLAN_OLD_PRI_PROP}`
	sip_dscp=`getprop ${VLAN_OLD_SIP_DSCP}`
	audio_dscp=`getprop ${VLAN_OLD_AUDIO_DSCP}`
	video_dscp=`getprop ${VLAN_OLD_VIDEO_DSCP}`
	
	saved=`getprop ${NET_SAVE_PROP}`
	stored=`getprop ${NET_STORE_PROP}`
        if [ $saved -eq 0 -a $stored -eq 1 ];then
                nvram unset :51
		nvram unset :87
		nvram unset :1558
		nvram unset :1559
		nvram unset :1560
                #nvram unset :dscp1
                #nvram unset :dscp2
                #nvram unset :dscp3
                #nvram unset :dscp4
                #nvram unset :dscp5
                #nvram unset :dscp6
                #nvram unset :dscp7
                #nvram unset :dscp8

                nvram set 51=$vlan_tag
		nvram set 87=$qos_priority
		nvram set 1558=$sip_dscp
		nvram set 1559=$audio_dscp
		nvram set 1560=$video_dscp
		nvram commit
                send_cfupdate
                setprop $VLAN_OLD_PROP "0"
                setprop $VLAN_OLD_PRI_PROP "0"
                setprop $VLAN_OLD_AUDIO_DSCP "0"
                setprop $VLAN_OLD_SIP_DSCP "0"
                setprop $VLAN_OLD_VIDEO_DSCP "0"
                setprop $NET_STORE_PROP "0"
		echo "restore vlan_tag=${vlan_tag}, qos_priority=${qos_priority},sip_dscp=${sip_dscp},audio_dscp=${audio_dscp},video_dscp=${video_dscp}"  >> /tmp/lldp.log
        else
                setprop ${NET_SAVE_PROP} "0"
        fi
}

join_med_vlan(){
        vlan_tag=$1
        qos_priority=$2
        dscp=$3
        setprop ${NET_SAVE_PROP} "0"
        setprop $VLAN_PROP $vlan_tag
        setprop $VLAN_PRI_PROP ${qos_priority} 

	nvram set :51=$1
	nvram set :87=$2

        case "$4" in
            1) nvram set :dscp1=$3
               nvram set :1559=$3
               ;;
            2) nvram set :dscp2=$3
               nvram set :1558=$3
               ;;
            3) nvram set :dscp3=$3
               ;;
            4) nvram set :dscp4=$3
               ;;
            5) nvram set :dscp5=$3
               ;;
            6) nvram set :dscp6=$3
               ;;
            7) nvram set :dscp7=$3
               nvram set :1560=$3
               ;;
            8) nvram set :dscp8=$3
               ;;
        esac
        send_cfupdate
}

leave_med_vlan(){
        setprop $VLAN_PROP "0"
        setprop $VLAN_PRI_PROP "0"
}


#LLDP_TAG=`nvram get :in_lldp`

if [ $ENABLE -eq 1 ]; then
	if [ $DTYPE -eq 4 -a $ATYPE -gt 0 ]; then
		if [ $TAGGED -eq 1 ]; then
			med_need_join ${VID}
			if [ 1 -eq $? ]; then
				store_cfg 
				join_med_vlan ${VID} ${PRIORITY} ${DSCP} ${ATYPE}
				nvram set :in_lldp="1"
			fi
		else
                        LAST_VID=`nvram get 51`
			med_need_leave ${LAST_VID}
			if [ 1 -eq $? ]; then
				restore_cfg
				leave_med_vlan ${VID}
				nvram set :in_lldp="0"
			fi
		fi
		lldpctl -P ${MED_NP} &
        fi
else 
        if [ $DTYPE -eq 4 -a $ATYPE -gt 0 ]; then
		LAST_VID=`nvram get 51`
		med_need_leave ${LAST_VID}
		if [ 1 -eq $? ]; then
			restore_cfg
			leave_med_vlan 
			nvram set :in_lldp="0"
		fi
		MED_NP="${ATYPE}:1:0:0:0:0"
		lldpctl -P ${MED_NP} &
        fi
fi
