/*
 * Copyright (c) 2011 ygzhang <ygzhang@grandstream.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* @20110920 dynamic load nvram module */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <dlfcn.h>
#include "nvram_mod.h"
#include "lldpd.h"
#include <fcntl.h>
int nvr_mod_init(const char *lib_fname, const char *cfg_fname, nvr_mod_t **pp_mod)
{
	int rval = 0;
	nvr_mod_t *mod = NULL;
	void *hdl = NULL;

	if(!lib_fname) lib_fname = "/system/lib/libnvram.so";
	hdl = dlopen(lib_fname, RTLD_NOW);

	if(!hdl)
	{
		log_warn("dlopen err");
		return -errno;
	}

	mod = calloc(1, sizeof(nvr_mod_t));
	if(!mod)
	{
		log_warn("calloc err");
		rval = -errno;
		goto err_alloc;
	}

	mod->hdl = hdl;
	mod->nv_init = dlsym(hdl, "nvram_init");
	mod->nv_exit = dlsym(hdl, "nvram_exit");
	mod->nv_get = dlsym(hdl, "nvram_get");
	mod->nv_set = dlsym(hdl, "nvram_set");
	mod->nv_commit = dlsym(hdl, "nvram_commit");

	log_warn("%s %s %s %s %s not found", 
		mod->nv_init?"":"nv_init", mod->nv_exit?"":"nv_exit",
		mod->nv_get?"":"nv_get", mod->nv_set?"":"nv_set",
		mod->nv_commit?"":"nv_commit");
	if(!mod->nv_init)
	{
		rval = -1;
		goto err_dlsym;
	}
	
	rval = mod->nv_init(NULL);
	if(rval <0)
	{
		log_warn("nv_init err");
		goto err_dlsym;
	}

	snprintf(mod->key_hd_ver, sizeof(mod->key_hd_ver), "%s", HD_VER_KEY);
	snprintf(mod->key_btlr_ver, sizeof(mod->key_btlr_ver), "%s", BTLR_VER_KEY);
	snprintf(mod->key_core_ver, sizeof(mod->key_core_ver), "%s", CORE_VER_KEY);
	snprintf(mod->key_base_ver, sizeof(mod->key_base_ver), "%s", BASE_VER_KEY);
	snprintf(mod->key_prog_ver, sizeof(mod->key_prog_ver), "%s", PROG_VER_KEY);
	snprintf(mod->key_guia_ver, sizeof(mod->key_guia_ver), "%s", GUIA_VER_KEY);
	snprintf(mod->key_guib_ver, sizeof(mod->key_guib_ver), "%s", GUIB_VER_KEY);
	snprintf(mod->key_rcvr_ver, sizeof(mod->key_rcvr_ver), "%s", RCVR_VER_KEY);
	snprintf(mod->key_supp_ver, sizeof(mod->key_supp_ver), "%s", SUPP_VER_KEY);
	snprintf(mod->key_sn, sizeof(mod->key_sn), "%s", SN_KEY);
	snprintf(mod->key_vendor, sizeof(mod->key_vendor), "%s", VENDOR_KEY);
	snprintf(mod->key_vendor_f, sizeof(mod->key_vendor_f), "%s", VENDOR_KEY_F);
	snprintf(mod->key_model, sizeof(mod->key_model), "%s", MODEL_KEY);
	snprintf(mod->key_asset, sizeof(mod->key_asset), "%s", ASSET_KEY);

	log_info("nvr mod init %p", hdl);
	log_info("  key_hd_ver: <%s>", mod->key_hd_ver);
	log_info("key_btlr_ver: <%s>", mod->key_btlr_ver);
	log_info("key_core_ver: <%s>", mod->key_core_ver);
	log_info("key_base_ver: <%s>", mod->key_base_ver);
	log_info("key_prog_ver: <%s>", mod->key_prog_ver);
	log_info("key_guia_ver: <%s>", mod->key_guia_ver);
	log_info("key_guib_ver: <%s>", mod->key_guib_ver);
	log_info("key_rcvr_ver: <%s>", mod->key_rcvr_ver);
	log_info("key_supp_ver: <%s>", mod->key_supp_ver);
	log_info("    key_sn: <%s>", mod->key_sn);
	log_info("key_vendor: <%s>", mod->key_vendor);
	log_info("key_vendor_f: <%s>", mod->key_vendor_f);
	log_info(" key_model: <%s>", mod->key_model);
	log_info(" key_asset: <%s>", mod->key_asset);
	*pp_mod = mod;
	return 0;

err_dlsym:
	free(mod);
err_alloc:
	dlclose(hdl);
	return rval;
}

int nvr_mod_exit(nvr_mod_t *mod)
{
	if(!mod)
	{
		return 0;
	}

	if(mod->nv_exit)
	{
		log_debug("nvr exit");
		mod->nv_exit();
	}

	if(mod->hdl)
	{
		log_debug("nvr dlclose %p", mod->hdl);
		dlclose(mod->hdl);
	}

	log_debug("free nvr mod");
	free(mod);

	return 0;
}

