/*
 * Copyright (c) 2011 ygzhang <ygzhang@grandstream.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _NVRAM_MOD_H
#define _NVRAM_MOD_H

#define HD_VER_KEY		":89"
#define BTLR_VER_KEY	":69"
#define CORE_VER_KEY	":70"
#define BASE_VER_KEY	":45"
#define PROG_VER_KEY	":68"
#define GUIA_VER_KEY	":7030"
#define GUIB_VER_KEY	":7031"
#define RCVR_VER_KEY	":7033"
#define SUPP_VER_KEY	":7034"
#define SN_KEY			":med_sn"
#define VENDOR_KEY		":vendor_name"
#define VENDOR_KEY_F		":vendor_fullname"
#define MODEL_KEY		":base_model"
#define ASSET_KEY		":asset"

#define MED_POE_DTYPE	":med_poe_dtype"
#define MED_POE_SRC	":med_poe_source"
#define MED_POE_PRIO	":med_poe_priority"
#define MED_POE_VAL	":med_poe_value"

#define SYS_DESC_KEY1	":dev_alias"
#define SYS_DESC_KEY2	PROG_VER_KEY

typedef int(* nv_init_t)(void *);
typedef void(* nv_exit_t)(void);
typedef char *(* nv_get_t)(const char *name);
typedef int(* nv_set_t)(const char *name, const char *value);
typedef int(* nv_commit_t)(void);

typedef struct
{
	void *hdl;
	nv_init_t nv_init;
	nv_exit_t nv_exit;
	nv_get_t nv_get;
	nv_set_t nv_set;
	nv_commit_t nv_commit;
	char key_hd_ver[32];
	char key_btlr_ver[32];
	char key_core_ver[32];
	char key_base_ver[32];
	char key_prog_ver[32];
	char key_guia_ver[32];
	char key_guib_ver[32];
	char key_rcvr_ver[32];
	char key_supp_ver[32];
	char key_sn[32];
	char key_vendor[32];
	char key_vendor_f[32];
	char key_model[32];
	char key_asset[32];
}nvr_mod_t;

int nvr_mod_init(const char *lib_fname, const char *cfg_fname, nvr_mod_t **pp_mod);
int nvr_mod_exit(nvr_mod_t *mod);

#endif /* _NVRAM_MOD_H */
